@@ -417,6 +417,35 @@ device_list_nodes (DBusGProxy *device, BluetoothClient *client, gboolean connect
 	return table;
 }
 
+static char **
+device_list_uuids (GValue *value)
+{
+	GPtrArray *ret;
+	char **uuids;
+	guint i;
+
+	if (value == NULL)
+		return NULL;
+
+	uuids = g_value_get_boxed (value);
+	if (uuids == NULL)
+		return NULL;
+
+	ret = g_ptr_array_new ();
+
+	for (i = 0; uuids[i] != NULL; i++) {
+		char *uuid;
+
+		//FIXME convert to bluetooth short names
+		uuid = g_strdup (uuids[i]);
+		g_ptr_array_add (ret, uuid);
+	}
+
+	g_ptr_array_add (ret, NULL);
+
+	return (char **) g_ptr_array_free (ret, FALSE);
+}
+
 static void device_changed(DBusGProxy *device, const char *property,
 					GValue *value, gpointer user_data)
 {
@@ -461,10 +490,14 @@ static void device_changed(DBusGProxy *device, const char *property,
 				BLUETOOTH_COLUMN_CONNECTED, connected, -1);
 	} else if (g_str_equal (property, "UUIDs") == TRUE) {
 		GHashTable *services;
+		char **uuids;
 
 		services = device_list_nodes (device, client, TRUE);
+		uuids = device_list_uuids (value);
 		gtk_tree_store_set(priv->store, &iter,
-				   BLUETOOTH_COLUMN_SERVICES, services, -1);
+				   BLUETOOTH_COLUMN_SERVICES, services,
+				   BLUETOOTH_COLUMN_UUIDS, uuids, -1);
+		g_strfreev (uuids);
 	}
 }
 
@@ -476,6 +509,7 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 	DBusGProxy *device;
 	GValue *value;
 	const gchar *address, *alias, *name, *icon;
+	char **uuids;
 	GHashTable *services;
 	gboolean paired, trusted, connected;
 	guint type;
@@ -519,6 +553,9 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 
 		value = g_hash_table_lookup(hash, "Connected");
 		connected = value ? g_value_get_boolean(value) : FALSE;
+
+		value = g_hash_table_lookup(hash, "UUIDs");
+		uuids = device_list_uuids (value);
 	} else {
 		if (device)
 			g_object_unref (device);
@@ -542,6 +579,7 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 					BLUETOOTH_COLUMN_TYPE, type,
 					BLUETOOTH_COLUMN_ICON, icon,
 					BLUETOOTH_COLUMN_RSSI, rssi,
+					BLUETOOTH_COLUMN_UUIDS, uuids,
 					-1);
 
 			if (device != NULL) {
@@ -577,6 +615,7 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 				BLUETOOTH_COLUMN_TRUSTED, trusted,
 				BLUETOOTH_COLUMN_CONNECTED, connected,
 				BLUETOOTH_COLUMN_SERVICES, services,
+				BLUETOOTH_COLUMN_UUIDS, uuids,
 				-1);
 
 done:
@@ -587,6 +626,7 @@ done:
 				G_CALLBACK(device_changed), client, NULL);
 		g_object_unref(device);
 	}
+	g_strfreev (uuids);
 }
 
 static void device_found(DBusGProxy *adapter, const char *address,
@@ -641,58 +681,7 @@ static void adapter_changed(DBusGProxy *adapter, const char *property,
 	if (get_iter_from_proxy(priv->store, &iter, adapter) == FALSE)
 		return;
 
-	if (g_str_equal (property, "Powered") == TRUE) {
-		GHashTable *hash;
-		const gchar *address, *name;
-		gboolean discovering, powered;
-		GPtrArray *array;
-
-		/* Need to update those properties! */
-		adapter_get_properties(adapter, &hash, NULL);
-		if (hash != NULL) {
-			gboolean is_default;
-
-			value = g_hash_table_lookup(hash, "Address");
-			address = value ? g_value_get_string(value) : NULL;
-
-			value = g_hash_table_lookup(hash, "Name");
-			name = value ? g_value_get_string(value) : NULL;
-
-			value = g_hash_table_lookup(hash, "Discovering");
-			discovering = value ? g_value_get_boolean(value) : FALSE;
-
-			value = g_hash_table_lookup(hash, "Powered");
-			powered = value ? g_value_get_boolean(value) : FALSE;
-
-			gtk_tree_store_set(priv->store, &iter,
-					   BLUETOOTH_COLUMN_ADDRESS, address,
-					   BLUETOOTH_COLUMN_NAME, name,
-					   BLUETOOTH_COLUMN_DISCOVERING, discovering,
-					   BLUETOOTH_COLUMN_POWERED, powered,
-					   -1);
-			notify = TRUE;
-
-			/* Update the power setting */
-			gtk_tree_model_get(GTK_TREE_MODEL (priv->store), &iter,
-					   BLUETOOTH_COLUMN_DEFAULT, &is_default,
-					   -1);
-			if (is_default) {
-				priv->default_adapter_powered = powered;
-				g_object_notify (G_OBJECT (client), "default-adapter-powered");
-			}
-
-			adapter_list_devices(adapter, &array, NULL);
-			if (array != NULL) {
-				int i;
-
-				for (i = 0; i < array->len; i++) {
-					gchar *path = g_ptr_array_index(array, i);
-					device_created(adapter, path, client);
-					g_free(path);
-				}
-			}
-		}
-	} else if (g_str_equal(property, "Name") == TRUE) {
+	if (g_str_equal(property, "Name") == TRUE) {
 		const gchar *name = g_value_get_string(value);
 
 		gtk_tree_store_set(priv->store, &iter,
@@ -722,7 +711,7 @@ static void adapter_added(DBusGProxy *manager,
 	BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(client);
 	GtkTreeIter iter;
 	DBusGProxy *adapter;
-	GPtrArray *array = NULL;
+	GPtrArray *devices;
 	GHashTable *hash = NULL;
 	GValue *value;
 	const gchar *address, *name;
@@ -746,11 +735,15 @@ static void adapter_added(DBusGProxy *manager,
 
 		value = g_hash_table_lookup(hash, "Powered");
 		powered = value ? g_value_get_boolean(value) : FALSE;
+
+		value = g_hash_table_lookup(hash, "Devices");
+		devices = value ? g_value_get_boxed (value) : NULL;
 	} else {
 		address = NULL;
 		name = NULL;
 		discovering = FALSE;
 		powered = FALSE;
+		devices = NULL;
 	}
 
 	gtk_tree_store_insert_with_values(priv->store, &iter, NULL, -1,
@@ -781,12 +774,11 @@ static void adapter_added(DBusGProxy *manager,
 	dbus_g_proxy_connect_signal(adapter, "DeviceFound",
 				G_CALLBACK(device_found), client, NULL);
 
-	adapter_list_devices(adapter, &array, NULL);
-	if (array != NULL) {
+	if (devices != NULL) {
 		int i;
 
-		for (i = 0; i < array->len; i++) {
-			gchar *path = g_ptr_array_index(array, i);
+		for (i = 0; i < devices->len; i++) {
+			gchar *path = g_ptr_array_index(devices, i);
 			device_created(adapter, path, client);
 			g_free(path);
 		}
@@ -927,7 +919,7 @@ static void bluetooth_client_init(BluetoothClient *client)
 					 G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INT,
 					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
 					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
-					 G_TYPE_HASH_TABLE);
+					 G_TYPE_HASH_TABLE, G_TYPE_STRV);
 
 	priv->dbus = dbus_g_proxy_new_for_name(connection, DBUS_SERVICE_DBUS,
 				DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS);
